/*
 * Copyright (c) 1997-2004  Motoyuki Kasahara
 * Modyfied by T. Nemoto (2005) for Zten Variant
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "build-pre.h"
#include <eb/eb.h>
#include <eb/error.h>
#include <eb/text.h>
#include "ztentext.h"
#include "build-post.h"

ZTEN_Hookset zten_default_hookset;

/*
 * Initialize a hookset.
 */
void
zten_initialize_hookset(ZTEN_Hookset *hookset)
{
    int i;

    LOG(("in: zten_initialize_hookset()"));

    eb_initialize_lock(&hookset->lock);

    for (i = 0; i < ZTEN_NUMBER_OF_HOOKS; i++) {
	hookset->hooks[i].code = i;
	hookset->hooks[i].function = NULL;
    }
    hookset->hooks[EB_HOOK_NARROW_JISX0208].function
	= eb_hook_euc_to_ascii;
    hookset->hooks[EB_HOOK_NARROW_FONT].function
	= eb_hook_narrow_character_text;
    hookset->hooks[EB_HOOK_WIDE_FONT].function
	= eb_hook_wide_character_text;
    hookset->hooks[EB_HOOK_NEWLINE].function
	= eb_hook_newline;

    LOG(("out: zten_initialize_hookset()"));
}


/*
 * Finalize a hookset.
 */
void
zten_finalize_hookset(ZTEN_Hookset *hookset)
{
    int i;

    LOG(("in: zten_finalize_hookset()"));

    for (i = 0; i < ZTEN_NUMBER_OF_HOOKS; i++) {
	hookset->hooks[i].code = i;
	hookset->hooks[i].function = NULL;
    }
    eb_finalize_lock(&hookset->lock);

    LOG(("out: zten_finalize_hookset()"));
}



/*
 * Set a hook.
 */
EB_Error_Code
zten_set_hook(ZTEN_Hookset *hookset, const EB_Hook *hook)
{
    EB_Error_Code error_code;

    eb_lock(&hookset->lock);
    LOG(("in: zten_set_hook(hook=%d)", (int)hook->code));

    /*
     * Set a hook.
     */
    if (hook->code < 0 || ZTEN_NUMBER_OF_HOOKS <= hook->code) {
	error_code = EB_ERR_NO_SUCH_HOOK;
	goto failed;
    }
    hookset->hooks[hook->code].function = hook->function;

    LOG(("out: zten_set_hook() = %s", eb_error_string(EB_SUCCESS)));
    eb_unlock(&hookset->lock);

    return EB_SUCCESS;

    /*
     * An error occurs...
     */
  failed:
    LOG(("out: zten_set_hook() = %s", eb_error_string(error_code)));
    eb_unlock(&hookset->lock);
    return error_code;
}


/*
 * Set a list of hooks.
 */
EB_Error_Code
zten_set_hooks(ZTEN_Hookset *hookset, const EB_Hook *hook)
{
    EB_Error_Code error_code;
    const EB_Hook *h;

    eb_lock(&hookset->lock);
    LOG(("in: zten_set_hooks(hooks=[below])"));

    if (eb_log_flag) {
	for (h = hook; h->code != EB_HOOK_NULL; h++)
	    LOG(("    hook=%d", h->code));
    }

    /*
     * Set hooks.
     */
    for (h = hook; h->code != EB_HOOK_NULL; h++) {
	if (h->code < 0 || ZTEN_NUMBER_OF_HOOKS <= h->code) {
	    error_code = EB_ERR_NO_SUCH_HOOK;
	    goto failed;
	}
	hookset->hooks[h->code].function = h->function;
    }

    /*
     * Unlock the hookset.
     */
    LOG(("out: zten_set_hooks() = %s", eb_error_string(EB_SUCCESS)));
    eb_unlock(&hookset->lock);

    return EB_SUCCESS;

    /*
     * An error occurs...
     */
  failed:
    LOG(("out: zten_set_hooks() = %s", eb_error_string(error_code)));
    eb_unlock(&hookset->lock);
    return error_code;
}


